@@ -1,6 +1,6 @@
-NLF.LQL <- function (params.fitted, object, params, par.index, transform.params = FALSE,
-                     times, t0, lags, period, tensor, seed = NULL, transform = identity,
-                     nrbf = 4, verbose = FALSE,
+NLF.LQL <- function (params.fitted, object, params, par.index, transform = FALSE,
+                     times, t0, lags, period, tensor, seed = NULL,
+                     transform.data = identity, nrbf = 4, verbose = FALSE,
                      bootstrap = FALSE, bootsamp = NULL) {
 
 ###>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
@@ -9,16 +9,14 @@ NLF.LQL <- function (params.fitted, object, params, par.index, transform.params
 ### so a large NEGATIVE value is used to flag bad parameters 
 ###>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 
-  transform.params <- as.logical(transform.params)
+  transform <- as.logical(transform)
 
   FAILED =  -99999999999
   params[par.index] <- params.fitted
   
-  if (transform.params)
+  if (transform)
     params <- partrans(object,params,dir="forward")
 
-  ## Need to extract number of state variables (nvar) from pomp object
-  ## Need to include simulation times in problem specification
   ## Evaluates the NLF objective function given a POMP object.
   ## Version 0.1, 3 Dec. 2007, Bruce E. Kendall & Stephen P. Ellner
   ## Version 0.2, May 2008, Stephen P. Ellner  
@@ -31,6 +29,7 @@ NLF.LQL <- function (params.fitted, object, params, par.index, transform.params
            )
   if (inherits(y,"try-error"))
     stop(sQuote("NLF.LQL")," reports: error in simulation")
+
   ## Test whether the model time series is valid
   if (!all(is.finite(y))) return(FAILED)
 
@@ -38,8 +37,8 @@ NLF.LQL <- function (params.fitted, object, params, par.index, transform.params
                     dim=c(nrow(data.ts),length(times)),
                     dimnames=list(rownames(data.ts),NULL)
                     )
-  model.ts[,] <- apply(y[,1,,drop=FALSE],c(2,3),transform)
-  data.ts[,] <- apply(data.ts,2,transform)
+  model.ts[,] <- apply(y[,1,,drop=FALSE],c(2,3),transform.data)
+  data.ts[,] <- apply(data.ts,2,transform.data)
   
   LQL <- try(
              NLF.guts(
