@@ -570,8 +570,8 @@ pomp(
      zeronames=c("cases"),
      comp.names=c("S","I","R"),
      ic.names=c("S.0","I.0","R.0"),
-     parameter.transform="_sir_par_trans",
-     parameter.inv.transform="_sir_par_untrans",
+     fromEstimationScale="_sir_par_trans",
+     toEstimationScale="_sir_par_untrans",
      nbasis=3L,
      degree=3L,
      period=1.0,
