@@ -5,10 +5,10 @@
 #include "pomp.h" // in R, do 'system.file("include/pomp.h",package="pomp")' to find this header file
 
 // define some macros to make the code easier to read
-#define LOG_R       (p[parindex[0]]) // growth rate
-#define LOG_K       (p[parindex[1]]) // carrying capacity
-#define LOG_SIGMA   (p[parindex[2]]) // process noise level
-#define LOG_TAU     (p[parindex[3]]) // measurement noise level
+#define R       (p[parindex[0]]) // growth rate
+#define K       (p[parindex[1]]) // carrying capacity
+#define SIGMA   (p[parindex[2]]) // process noise level
+#define TAU     (p[parindex[3]]) // measurement noise level
 
 #define Y           (y[obsindex[0]])   // observed population size
 #define X           (x[stateindex[0]]) // actual population size
@@ -18,14 +18,14 @@
 void _gompertz_normal_dmeasure (double *lik, double *y, double *x, double *p, int give_log,
 				int *obsindex, int *stateindex, int *parindex, int *covindex,
 				int ncovars, double *covars, double t) {
-  *lik = dlnorm(Y,log(X),exp(LOG_TAU),give_log);
+  *lik = dlnorm(Y,log(X),TAU,give_log);
 }
 
 // normal measurement model simulator
 void _gompertz_normal_rmeasure (double *y, double *x, double *p, 
 				int *obsindex, int *stateindex, int *parindex, int *covindex,
 				int ncovars, double *covars, double t) {
-  Y = rlnorm(log(X),exp(LOG_TAU));
+  Y = rlnorm(log(X),TAU);
 }
 
 // stochastic Gompertz model with log-normal process noise
@@ -34,10 +34,9 @@ void _gompertz_simulator (double *x, const double *p,
 			  int covdim, const double *covar, 
 			  double t, double dt)
 {
-  double S = exp(-exp(LOG_R)*dt);
-  double sigma = exp(LOG_SIGMA);
-  double logeps = (sigma > 0.0) ? rnorm(0,sigma) : 0.0;
-  X = exp((1-S)*LOG_K)*pow(X,S)*exp(logeps); // note X is over-written by this line
+  double S = exp(-R*dt);
+  double eps = (SIGMA > 0.0) ? exp(rnorm(0,SIGMA)) : 1.0;
+  X = pow(K,(1-S))*pow(X,S)*eps; // note X is over-written by this line
 }
 
 // the deterministic skeleton
@@ -46,6 +45,6 @@ void _gompertz_skeleton (double *f, double *x, const double *p,
 			 int covdim, const double *covar, double t) 
 {
   double dt = 1.0;
-  double S = exp(-exp(LOG_R)*dt);
-  XPRIME = exp((1-S)*LOG_K)*pow(X,S); // X is not over-written in the skeleton function
+  double S = exp(-R*dt);
+  XPRIME = pow(K,(1-S))*pow(X,S); // X is not over-written in the skeleton function
 }
